from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime
from Field import MetadataField, LinesField, DateTimeField, MetadataFieldList, IntegerField
from Form import StringInfo, KeywordsInfo, DateTimeInfo, LanguageInfo, TextAreaInfo, LinesInfo, SingleSelectionInfo
from Globals import InitializeClass
from Products.CMFCore  import CMFCorePermissions
from Products.CMFCore.utils  import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from UserDict import UserDict
from ZODB.PersistentMapping import PersistentMapping
from debug import log
import Persistence

from utils import DisplayList



## MIXIN
class ExtensibleMetadata(DefaultDublinCoreImpl, Persistence.Persistent):
    security = ClassSecurityInfo()
    security.declareObjectPublic()
    security.setDefaultAccess('allow')
    
    type = MetadataFieldList((
        MetadataField('allowDiscussion',
                      accessor="isDiscussable",
                      mutator="allowDiscussion",
                      default=0,
                      vocabulary=DisplayList(((0, 'off'), (1, 'on'), (None, 'default'))),
                      form_info=SingleSelectionInfo(visible=-1,
                                                    format="radio",
                                                    label="Allow Discussions",
                                                    description="""
                                                    Specifies whether
                                                    discussion items
                                                    and comments can
                                                    be added
                                                    to this
                                                    item. 'Default'
                                                    will use the
                                                    site-wide setting
                                                    for  Discussion.""")), 

              
        LinesField('subject',
                   multiValued=1,
                   form_info=KeywordsInfo(label="Keywords"),
                   ),
        MetadataField('description',
                      searchable=1,
                      form_info=TextAreaInfo(description="An administrative summary of the content")),

        LinesField('contributors',
                   form_info=LinesInfo()),
        DateTimeField('effectiveDate',
                      editable="EffectiveDate",
                      form_info=DateTimeInfo(description="Date when the content should become availble on the public site", )),
        DateTimeField('expirationDate',
                      editable="ExpirationDate",
                      form_info=DateTimeInfo(description="Date when the content should no longer be visible on the public site", )),
        MetadataField('language',
                      default="en-US",
                      form_info=LanguageInfo()),
        MetadataField('rights',
                      form_info=StringInfo(description="A list of copyright info for this content")),
        
     ))

    def isDiscussable(self):
        result = 0
        try:
            result = getToolByName(self, 'portal_discussion').isDiscussionAllowedFor(self)
        except:
            pass
        return result
    
    def allowDiscussion(self, allowDiscussion=None):
        if allowDiscussion:
            allowDiscussion = allowDiscussion.lower().strip()

            try:
                allowDiscussion = int(allowDiscussion)
            except:
                allowDiscussion = {'on' : 1, 'off': 0}.get(allowDiscussion, 0)

            try:
                getToolByName(self, 'portal_discussion').overrideDiscussionFor(self, allowDiscussion)
            except:
                log_exc()
                pass
            
    


InitializeClass(ExtensibleMetadata)
