from debug import log


                       
class FormInfo:
    """
    description -- tooltip
    label       -- textual label
    visible     -- 1[default] visible 0 hidden -1 skipped
    """
    
    validator = "String"
    def __init__(self, **kwargs):
        self._processed  = 0
        self.description = kwargs.get('description', None)
        self.label       = kwargs.get('label', None)
        self.visible     = kwargs.get('visible', 1)
        self.attributes  = kwargs.get('attributes', '')

        self._process_args(**kwargs)
        
    def _process_args(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

class StringInfo(FormInfo):
    """
    Basic String Type
    max_length  -- None
    """
    validator = "String"


class PasswordInfo(FormInfo):
    def __init__(self, **kwargs):
        FormInfo.__init__(self, **kwargs)
        self.confirm = kwargs.get('confirm', 1)
        

class TextAreaInfo(FormInfo):
    validator = "Text"
    def __init__(self, **kwargs):
        FormInfo.__init__(self, **kwargs)
        self.rows = kwargs.get('rows', 5)
        self.columns = kwargs.get('columns', 40)

class LinesInfo(TextAreaInfo):
    validator = "Lines"

class KeywordsInfo(LinesInfo):
    def __init__(self, **kwargs):
        LinesInfo.__init__(self, **kwargs)
        self.index = kwargs.get('index', 'Subject')

class LanguageInfo(FormInfo):
    pass

class BooleanInfo(FormInfo):
    pass

    
class DateTimeInfo(FormInfo):
    validator = "DateTime"

class LinkInfo(FormInfo):
    validator = "Link"

class FileInfo(FormInfo):
    # We may never want this one used
    validator = "File"

class IntegerInfo(FormInfo):
    validator = "Integer"

class FloatInfo(FormInfo):
    validator = "Float"

class EmailInfo(FormInfo):
    validator = "Email"
    

class RichFormInfo(TextAreaInfo):
    validator = "File" ## Here we might wanna hook into formulators external
                       ## and call something on the object as this could
                       ## be a set of file-types with different options
    
class SingleSelectionInfo(FormInfo):
    validator = None                              
    def __init__(self, **kwargs):
        FormInfo.__init__(self, **kwargs)
        self.format = kwargs.get('format', 'radio') 
        # make sure they picked a valid format
        validFormats = ('radio', 'pulldown')
        if self.format not in validFormats:
            log("Unrecognized format (%s).  Defaulting to 'radio'" % self.format)
            self.format = 'radio'


class MultiSelectionInfo(FormInfo):
    validator = None
    def __init__(self, **kwargs):
        FormInfo.__init__(self, **kwargs)
        self.format = 'multi'
        self.size   = int(kwargs.get('size', '4'))
            


class SlotInfo(FormInfo):
    validator = None
    def __init__(self, **kwargs):
        FormInfo.__init__(self, **kwargs)
        self.slot_metal = kwargs.get("slot_metal")
        
class IdInfo(FormInfo):
    pass

