from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.CMFCore  import CMFCorePermissions
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from copy import deepcopy
from types import StringType
import os.path
import sys

from Products.CMFTypes  import PKG_NAME
from ClassGen import generateClass
from Generator import generateViews
from debug import log, log_exc
from utils import pathFor, capitalize, findDict

# This is the template that we produce our custom types from
# Never actually used
base_factory_type_information = (
    { 'id': 'ContentFactory'
      ,  'content_icon': 'document_icon.gif'
      , 'meta_type': 'ContentFactory'
      , 'description': ( 'ContentFactory for flexible types')
      , 'product': 'Unknown Package'
      , 'factory': 'addContent'
      , 'immediate_view': 'base_edit_form'
      , 'actions': ( { 'id': 'references'
                       , 'name': 'References'
                       , 'action': 'portal_form/reference_edit'
                       , 'permissions': (CMFCorePermissions.ModifyPortalContent,)
                       },
                     ) 
      }, )


def modify_fti(fti, klass, pkg_name, actions):
    fti[0]['id']          = klass.__name__
    fti[0]['meta_type']   = klass.meta_type
    fti[0]['description'] = klass.__doc__
    fti[0]['factory']     = "add%s" % capitalize(klass.__name__)
    fti[0]['product']     = pkg_name

    if hasattr(klass, "icon"):
        icon = klass.icon
        try:
            icon = icon()
        except:
            pass
        if type(icon) is StringType:
            fti[0]['content_icon'] = icon

    if hasattr(klass, "allowed_content_types"):
        allowed = klass.allowed_content_types
        fti[0]['allowed_content_types'] = allowed
        
        
    # Take all the actions the generator setup
    # for all the modes having actions
    fti[0]['actions'] = actions + fti[0]['actions']

    # Update edit to use Plone's portal_form
    edit = findDict(fti[0]['actions'], 'id', 'edit')
    if edit:
        edit['action'] = "portal_form/%s" % edit['action']
        fti[0]['immediate_view'] = edit['action']

    md = findDict(fti[0]['actions'], 'id', 'metadata')
    if md:
        md['action'] = "portal_form/%s" % md['action']

def process_types(types,
                  pkg_name,
                  target_dir=None):
    content_types = ()
    constructors  = ()
    ftis          = ()

    if not target_dir:
        target_dir = pathFor(os.path.join("skins", "cmft"))
        
    for klass in types:
        module = sys.modules[klass.__module__]
        type   = klass.__name__

        generateClass(klass)
        actions = generateViews(klass, target_dir)

        if hasattr(module, "factory_type_information"):
            fti = module.factory_type_information
        else:
            fti = deepcopy(base_factory_type_information)
            modify_fti(fti, klass, pkg_name, actions)

        # Add a callback to modifty the fti
        if hasattr(module, "modify_fti"):
            module.modify_fti(fti)
            
        ctor   = getattr(module, "add%s" % capitalize(type),
                         lambda *args: None)
        
        content_types += (klass,)
        constructors  += (ctor,)
        ftis   += fti

    return content_types, constructors, ftis


_types = {}

def _guessPackage(base):
    if base.startswith('Products'):
        base = base[9:]
        idx = base.index('.')
        if idx != -1:
            base = base[:idx]
    return base

    
def registerType(type, package=None):
    if not package: package = _guessPackage(type.__module__)
    data = {
        'klass' : type,
        'name'  : type.meta_type,
        'package' : package,
        'type' : type.type,
        }
    
    _types[type.meta_type] = data

def listTypes(package=None):
    values = _types.values()
    if package:
        values = [v for v in values if v['package'] == package]
        
    return values

def getType(name):
    return _types[name]


class GeneratorTool(UniqueObject, ActionProviderBase, SimpleItem):
    """ Types Generation Types  """
    id        = 'generator_tool'
    meta_type = 'CMFTypes Generator Tool'


    manage_options=(
        ({ 'label'  : 'Introspect',
           'action' : 'manage_debugForm',
           },
         ) + 
        SimpleItem.manage_options #+
        # ({  'label'  : 'Generate',
        #     'action' : 'manage_generateForm',
        #     },        
        #  )
        )

    manage_generateForm = PageTemplateFile('www/generateView',
                                           globals())
    
    
    manage_debugForm = PageTemplateFile('www/generateDebug',
                                        globals())
        

    def listRegisteredTypes(self):
        """Return the list of sorted types"""
        def type_sort(a, b):
            v = cmp(a['package'], b['package'])
            if v != 0: return v
            c = cmp(a['type'].__class__.__name__,
                    b['type'].__class__.__name__)

            if c == 0:
                return cmp(a['package'], b['package'])
            return c
            
        values = listTypes()
        values.sort(type_sort)
        return values

    def lookupType(self, package, type):
        types = self.listRegisteredTypes()
        for t in types:
            if t['package'] != package: continue
            if t['name'] == type:
                return t
        return None
        
    def _generate(self, package, type):
        ti = self.lookupType(package, type)
        if not ti: return
        # We have the class and package names
        type = ti['klass']
        module = sys.modules[type.__module__]
        reload(module)
        #ti = self.lookupType(package, type)
        #type = ti['klass']
        types = (type,)
        process_types(types, package)
        
        
    def manage_doGenerate(self, submit, types=(), REQUEST=None, *args, **kwargs):
        """(Re)generate types """
        if submit == "Generate":
            for type in types:
                package, type = type.split(':', 2)
                self._generate(package, type)
        if REQUEST:
            return REQUEST.RESPONSE.redirect(self.absolute_url() + "/manage_workspace")

    
InitializeClass(GeneratorTool)




